package livelock;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Esempio di classe da adoperare nella concorrenza di thread
 * Gestione del deadlock con i blocchi 
 * Gestione del livelock implementando LiveLockSolution
 * @author Luciano Giuseppe
 */
public class Prodotto implements LiveLockSolution{
		private int quantit;
		private boolean endLoop;
		private ReentrantLock lock;
		private Condition cond;
		
		public Prodotto(ReentrantLock lock) {
			this.endLoop = false;
			this.quantit = 0;
			this.lock = lock;
			this.cond = lock.newCondition();
		}
		
		
		/**
		 * Produco un prodotto
		 */
		public void produci() {
			lock.lock();
			try {
				this.quantit++;
				System.out.println("Q:" + this.quantit);
				cond.signalAll();
			} finally {
				lock.unlock();
			}		
		}
		
		/**
		 * Consumo il prodotto, se questo non  disponibile allora attendo
		 */
		public void consuma() {
			lock.lock();
			try {
				while(endLoop == false && quantit == 0) {
					cond.await();
				}
				if(endLoop == false) { //se son stato avvisato non faccio niente
					this.quantit--;
					System.out.println("Q:" + this.quantit);
				}
				cond.signalAll();
			} catch(Exception ex) {
				ex.printStackTrace();
			} finally {
				lock.unlock();
			}
		}
		
		/**
		 * Faccio in modo da far terminare i vari thread che usano Prodotto e che stanno aspettando 
		 */
		@Override
		public void liveLockSolution() {
			lock.lock();
			try {
				this.endLoop = true; //dico a consuma di non aspettare altra produzione
				cond.signalAll();
			} finally {
				lock.unlock();
			}
		}

	}